package com.persagy.dmp.rwd.edit.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.FuncidDef;
import com.persagy.dmp.rwd.edit.entity.QFuncidDef;
import com.persagy.dmp.rwd.edit.util.ClassCodeUtils;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import com.persagy.dmp.rwd.enums.FuncidDataType;
import com.persagy.dmp.rwd.model.ClassDefModel;
import com.persagy.dmp.rwd.model.FuncidDefModel;
import com.querydsl.core.types.dsl.BooleanExpression;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
public class FuncidDefService {

    @Autowired
    private ClassDefService classDefService;

    @Autowired
    private ClassCodeUtils classCodeUtils;

    @Autowired
    private CriteriaUtils criteriaUtils;

    /***
     * Description: 根据条件查询信息点
     * @param request : 请求参数
     * @return : com.persagy.common.web.ListResponse<com.persagy.dmp.rwd.model.FuncidDefModel>
     * @author : lijie
     * @date :2021/6/22 12:17
     * Update By lijie 2021/6/22 12:17
     */
    public ListResponse<FuncidDefModel> queryFuncid(JacksonCriteria request) {
        // 1.根据classCode查询符合条件的信息点
        ListResponse<FuncidDefModel> resp = new ListResponse<>();
        List<FuncidDefModel> list = null;
        boolean hasClassCode = request.getCriteria().has("classCode");
        if (hasClassCode) {
            list = queryWithClassCode(request);
        }
        // 2.2021年6月22日12:18:10的排序需求
        if (CollUtil.isNotEmpty(list)){
            sortFuncidForEdit(list);
        }

        resp.setData(list);
        return resp;
    }
    /***
     * Description: 对信息点列表进行排序
     * @param list : 信息点列表
     * @return : void
     * @author : lijie
     * @date :2021/6/22 12:19
     * Update By lijie 2021/6/22 12:19
     */
    private void sortFuncidForEdit(List<FuncidDefModel> list) {
        // 一级按照“技术参数-运行参数-设定参数-台账参数”的逻辑展示；

        // 二级按通用信息点、个性化信息点的excel表格进行排序展示；
        // 三级按照excel表格的逻辑进行排序展示
        list.sort((k1,k2)->{
            if (StrUtil.isBlank(k1.getFirstTag()) && StrUtil.isBlank(k2.getFirstTag())){
                int compareTo = StrUtil.blankToDefault(k1.getShareType(),"").compareTo(StrUtil.blankToDefault(k2.getShareType(),""));
                if (0==compareTo) {
                    return k1.getOrderSeq().compareTo(k2.getOrderSeq());
                }
                return compareTo;
            }
            if (StrUtil.isBlank(k2.getFirstTag())){
                k2.setFirstTag("");
            }
            if (StrUtil.isBlank(k1.getFirstTag())){
                k1.setFirstTag("");
            }
            int firstTagSort = k1.getFirstTag().compareTo(k2.getFirstTag());
            if (0==firstTagSort){
                int compareTo = StrUtil.blankToDefault(k1.getShareType(),"").compareTo(StrUtil.blankToDefault(k2.getShareType(),""));
                if (0==compareTo) {
                    return k1.getOrderSeq().compareTo(k2.getOrderSeq());
                }
                return compareTo;
            }
            return firstTagSort;
        });
    }

    public PagedResponse<FuncidDefModel> query(JacksonCriteria criteria) {
        // 预处理criteria
        classCodeUtils.prepareCriteriaForFuncid(criteria, "classCode");

        PagedResponse<FuncidDef> resp = criteriaUtils.query(QFuncidDef.funcidDef, this::parse, criteria);
        PagedResponse<FuncidDefModel> result = new PagedResponse<>();
        result.setCount(resp.getCount());
        List<FuncidDef> dataList = resp.getData();
        if (dataList != null && dataList.size() > 0) {
            List<FuncidDefModel> collect = dataList.stream().map(entity -> {
                FuncidDefModel model = entity.toModel();
                return model;
            }).collect(Collectors.toList());
            //排序
            if (CollUtil.isNotEmpty(collect)){
                sortFuncidForEdit(collect);
            }
            result.setData(collect);
        }
        return result;
    }

    private List<BooleanExpression> parse(ObjectNode object) {
        // TODO
        List<BooleanExpression> list = new LinkedList<>();
        QFuncidDef qt = QFuncidDef.funcidDef;
        list.addAll(criteriaUtils.parse(qt.category, object.get("category"), FuncidCategory.class));
        list.addAll(criteriaUtils.parse(qt.dataType, object.get("dataType"), FuncidDataType.class));
        return list;
    }

    private List<FuncidDefModel> queryWithClassCode(JacksonCriteria request) {
        JsonNode classCode = request.getCriteria().get("classCode");
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.getCriteria().set("code", classCode);
        criteria.setWithoutCount(true);
        ListResponse<ClassDefModel> classDefModelListResponse = classDefService.queryClass(criteria);
        List<ClassDefModel> classDefModelList = classDefModelListResponse.getData();
        List<FuncidDefModel> list = new LinkedList<>();
        if (classDefModelList.size() > 0) {
            for (ClassDefModel classDefModel : classDefModelList) {
                JacksonCriteria funcidCriteria = JacksonCriteria.newInstance();
                funcidCriteria.getCriteria().setAll(request.getCriteria());
                if (classDefModel.getParentCode() != null) {
                    funcidCriteria.add("classCode").in(Arrays.asList(classDefModel.getCode(), classDefModel.getParentCode()));
                } else {
                    funcidCriteria.add("classCode", classDefModel.getCode());
                }
                List<FuncidDefModel> funcidDefModels = queryWithoutClassCode(funcidCriteria);
                //funcidDefModels.forEach(model -> model.setClassCode(classDefModel.getCode()));
                list.addAll(funcidDefModels);
            }
        }
        return list;
    }

    private List<FuncidDefModel> queryWithoutClassCode(JacksonCriteria request) {
        Map<String, FuncidDefModel> cache = new HashMap<>();
        // 预处理criteria
        classCodeUtils.prepareCriteriaForFuncid(request, "classCode");

        ObjectNode requestCriteria = request.getCriteria();
        // 查询平台级
        JacksonCriteria criteria = JacksonCriteria.newInstance();
        criteria.getCriteria().setAll(requestCriteria);
        criteria.add("type", "common");
        criteria.add("groupCode", "0");
        criteria.setWithoutCount(true);
        PagedResponse<FuncidDefModel> queryResp = query(criteria);

        queryResp.getData().forEach(model -> cache.put(model.getClassCode() + "-" + model.getCode(), model));

        // 继承父级信息点
        List<FuncidDefModel> list = new LinkedList<>();
        list.addAll(cache.values());
        return list;
    }
}

