package com.persagy.dmp.rwd.edit.service;

import com.persagy.common.criteria.CriteriaExpressionParser;
import com.persagy.common.criteria.CriteriaUtils;
import com.persagy.common.criteria.JacksonCriteria;
import com.persagy.common.web.ListResponse;
import com.persagy.common.web.MapResponse;
import com.persagy.common.web.PagedResponse;
import com.persagy.dmp.rwd.edit.entity.FuncidDef;
import com.persagy.dmp.rwd.edit.entity.FuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.entity.QFuncidDefChangeRecord;
import com.persagy.dmp.rwd.edit.enumeration.EnumVersionState;
import com.persagy.dmp.rwd.edit.repository.FuncidDefChangeRecordRepository;
import com.persagy.dmp.rwd.edit.repository.FuncidDefRepository;
import com.persagy.dmp.rwd.enums.FuncidCategory;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;

class FuncidDefChangeRecordServiceTest {

    private FuncidDefChangeRecordService funcidDefChangeRecordServiceUnderTest;

    @BeforeEach
    void setUp() {
        funcidDefChangeRecordServiceUnderTest = new FuncidDefChangeRecordService();
        funcidDefChangeRecordServiceUnderTest.repository = mock(FuncidDefChangeRecordRepository.class);
        funcidDefChangeRecordServiceUnderTest.funcidDefService = mock(FuncidDefService.class);
        funcidDefChangeRecordServiceUnderTest.classDefService = mock(ClassDefService.class);
        funcidDefChangeRecordServiceUnderTest.rwdeditRefTypeInfosService = mock(RwdeditRefTypeInfosService.class);
        funcidDefChangeRecordServiceUnderTest.criteriaUtils = mock(CriteriaUtils.class);
        funcidDefChangeRecordServiceUnderTest.funcidDefRepository = mock(FuncidDefRepository.class);
    }

    @Test
    void testQuery() {
        // Setup
        final JacksonCriteria criteria = JacksonCriteria.newInstance();
        when(funcidDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QFuncidDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final PagedResponse<FuncidDefChangeRecord> result = funcidDefChangeRecordServiceUnderTest.query(criteria);

        // Verify the results
    }

    @Test
    void testCreate() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure FuncidDefChangeRecordRepository.findAll(...).
        final FuncidDefChangeRecord funcidDefChangeRecord = new FuncidDefChangeRecord();
        funcidDefChangeRecord.setId(0);
        funcidDefChangeRecord.setCode("code");
        funcidDefChangeRecord.setGroupCode("groupCode");
        funcidDefChangeRecord.setProjectId("projectId");
        funcidDefChangeRecord.setOrigCode("origCode");
        funcidDefChangeRecord.setName("name");
        funcidDefChangeRecord.setAliasCode("aliasCode");
        funcidDefChangeRecord.setAliasName("aliasName");
        funcidDefChangeRecord.setClassCode("classCode");
        funcidDefChangeRecord.setCategory(FuncidCategory.STATIC);
        final List<FuncidDefChangeRecord> funcidDefChangeRecords = Arrays.asList(funcidDefChangeRecord);
        when(funcidDefChangeRecordServiceUnderTest.repository.findAll()).thenReturn(funcidDefChangeRecords);

        // Configure FuncidDefChangeRecordRepository.save(...).
        final FuncidDefChangeRecord funcidDefChangeRecord1 = new FuncidDefChangeRecord();
        funcidDefChangeRecord1.setId(0);
        funcidDefChangeRecord1.setCode("code");
        funcidDefChangeRecord1.setGroupCode("groupCode");
        funcidDefChangeRecord1.setProjectId("projectId");
        funcidDefChangeRecord1.setOrigCode("origCode");
        funcidDefChangeRecord1.setName("name");
        funcidDefChangeRecord1.setAliasCode("aliasCode");
        funcidDefChangeRecord1.setAliasName("aliasName");
        funcidDefChangeRecord1.setClassCode("classCode");
        funcidDefChangeRecord1.setCategory(FuncidCategory.STATIC);
        when(funcidDefChangeRecordServiceUnderTest.repository.save(any(FuncidDefChangeRecord.class))).thenReturn(funcidDefChangeRecord1);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.create(param);

        // Verify the results
        verify(funcidDefChangeRecordServiceUnderTest.repository).deleteAll(Arrays.asList(new FuncidDefChangeRecord()));
    }

    @Test
    void testCreate_FuncidDefChangeRecordRepositoryFindAllReturnsNoItems() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.repository.findAll()).thenReturn(Collections.emptyList());

        // Configure FuncidDefChangeRecordRepository.save(...).
        final FuncidDefChangeRecord funcidDefChangeRecord = new FuncidDefChangeRecord();
        funcidDefChangeRecord.setId(0);
        funcidDefChangeRecord.setCode("code");
        funcidDefChangeRecord.setGroupCode("groupCode");
        funcidDefChangeRecord.setProjectId("projectId");
        funcidDefChangeRecord.setOrigCode("origCode");
        funcidDefChangeRecord.setName("name");
        funcidDefChangeRecord.setAliasCode("aliasCode");
        funcidDefChangeRecord.setAliasName("aliasName");
        funcidDefChangeRecord.setClassCode("classCode");
        funcidDefChangeRecord.setCategory(FuncidCategory.STATIC);
        when(funcidDefChangeRecordServiceUnderTest.repository.save(any(FuncidDefChangeRecord.class))).thenReturn(funcidDefChangeRecord);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.create(param);

        // Verify the results
        verify(funcidDefChangeRecordServiceUnderTest.repository).deleteAll(Arrays.asList(new FuncidDefChangeRecord()));
    }

    @Test
    void testUpdate() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        // Configure FuncidDefChangeRecordRepository.findById(...).
        final FuncidDefChangeRecord funcidDefChangeRecord1 = new FuncidDefChangeRecord();
        funcidDefChangeRecord1.setId(0);
        funcidDefChangeRecord1.setCode("code");
        funcidDefChangeRecord1.setGroupCode("groupCode");
        funcidDefChangeRecord1.setProjectId("projectId");
        funcidDefChangeRecord1.setOrigCode("origCode");
        funcidDefChangeRecord1.setName("name");
        funcidDefChangeRecord1.setAliasCode("aliasCode");
        funcidDefChangeRecord1.setAliasName("aliasName");
        funcidDefChangeRecord1.setClassCode("classCode");
        funcidDefChangeRecord1.setCategory(FuncidCategory.STATIC);
        final Optional<FuncidDefChangeRecord> funcidDefChangeRecord = Optional.of(funcidDefChangeRecord1);
        when(funcidDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(funcidDefChangeRecord);

        // Configure FuncidDefChangeRecordRepository.save(...).
        final FuncidDefChangeRecord funcidDefChangeRecord2 = new FuncidDefChangeRecord();
        funcidDefChangeRecord2.setId(0);
        funcidDefChangeRecord2.setCode("code");
        funcidDefChangeRecord2.setGroupCode("groupCode");
        funcidDefChangeRecord2.setProjectId("projectId");
        funcidDefChangeRecord2.setOrigCode("origCode");
        funcidDefChangeRecord2.setName("name");
        funcidDefChangeRecord2.setAliasCode("aliasCode");
        funcidDefChangeRecord2.setAliasName("aliasName");
        funcidDefChangeRecord2.setClassCode("classCode");
        funcidDefChangeRecord2.setCategory(FuncidCategory.STATIC);
        when(funcidDefChangeRecordServiceUnderTest.repository.save(any(FuncidDefChangeRecord.class))).thenReturn(funcidDefChangeRecord2);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.update(param);

        // Verify the results
    }

    @Test
    void testUpdate_FuncidDefChangeRecordRepositoryFindByIdReturnsAbsent() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        when(funcidDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(Optional.empty());

        // Configure FuncidDefChangeRecordRepository.save(...).
        final FuncidDefChangeRecord funcidDefChangeRecord = new FuncidDefChangeRecord();
        funcidDefChangeRecord.setId(0);
        funcidDefChangeRecord.setCode("code");
        funcidDefChangeRecord.setGroupCode("groupCode");
        funcidDefChangeRecord.setProjectId("projectId");
        funcidDefChangeRecord.setOrigCode("origCode");
        funcidDefChangeRecord.setName("name");
        funcidDefChangeRecord.setAliasCode("aliasCode");
        funcidDefChangeRecord.setAliasName("aliasName");
        funcidDefChangeRecord.setClassCode("classCode");
        funcidDefChangeRecord.setCategory(FuncidCategory.STATIC);
        when(funcidDefChangeRecordServiceUnderTest.repository.save(any(FuncidDefChangeRecord.class))).thenReturn(funcidDefChangeRecord);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.update(param);

        // Verify the results
    }

    @Test
    void testAddVersion() {
        // Setup
        when(funcidDefChangeRecordServiceUnderTest.repository.addVersion("version", Arrays.asList(0))).thenReturn(0);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.addVersion("version", Arrays.asList(0));

        // Verify the results
    }

    @Test
    void testDelete() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        // Configure FuncidDefChangeRecordRepository.findById(...).
        final FuncidDefChangeRecord funcidDefChangeRecord1 = new FuncidDefChangeRecord();
        funcidDefChangeRecord1.setId(0);
        funcidDefChangeRecord1.setCode("code");
        funcidDefChangeRecord1.setGroupCode("groupCode");
        funcidDefChangeRecord1.setProjectId("projectId");
        funcidDefChangeRecord1.setOrigCode("origCode");
        funcidDefChangeRecord1.setName("name");
        funcidDefChangeRecord1.setAliasCode("aliasCode");
        funcidDefChangeRecord1.setAliasName("aliasName");
        funcidDefChangeRecord1.setClassCode("classCode");
        funcidDefChangeRecord1.setCategory(FuncidCategory.STATIC);
        final Optional<FuncidDefChangeRecord> funcidDefChangeRecord = Optional.of(funcidDefChangeRecord1);
        when(funcidDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(funcidDefChangeRecord);

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.delete(param);

        // Verify the results
        verify(funcidDefChangeRecordServiceUnderTest.repository).delete(any(FuncidDefChangeRecord.class));
    }

    @Test
    void testDelete_FuncidDefChangeRecordRepositoryFindByIdReturnsAbsent() {
        // Setup
        final FuncidDefChangeRecord param = new FuncidDefChangeRecord();
        param.setId(0);
        param.setCode("code");
        param.setGroupCode("groupCode");
        param.setProjectId("projectId");
        param.setOrigCode("origCode");
        param.setName("name");
        param.setAliasCode("aliasCode");
        param.setAliasName("aliasName");
        param.setClassCode("classCode");
        param.setCategory(FuncidCategory.STATIC);

        when(funcidDefChangeRecordServiceUnderTest.repository.findById(0)).thenReturn(Optional.empty());

        // Run the test
        final MapResponse result = funcidDefChangeRecordServiceUnderTest.delete(param);

        // Verify the results
        verify(funcidDefChangeRecordServiceUnderTest.repository).delete(any(FuncidDefChangeRecord.class));
    }

    @Test
    void testGetSqlList() {
        // Setup
        when(funcidDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QFuncidDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.rwdeditRefTypeInfosService.query(any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());

        // Run the test
        final List<String> result = funcidDefChangeRecordServiceUnderTest.getSqlList("version", false);

        // Verify the results
        assertThat(result).isEqualTo(Arrays.asList("value"));
    }

    @Test
    void testUpdateStateByVersion() {
        // Setup
        when(funcidDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QFuncidDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure FuncidDefRepository.saveAll(...).
        final FuncidDef funcidDef = new FuncidDef();
        funcidDef.setId("id");
        funcidDef.setGroupCode("groupCode");
        funcidDef.setProjectId("projectId");
        funcidDef.setCode("code");
        funcidDef.setOrigCode("origCode");
        funcidDef.setName("name");
        funcidDef.setAliasCode("aliasCode");
        funcidDef.setAliasName("aliasName");
        funcidDef.setClassCode("classCode");
        funcidDef.setCategory(FuncidCategory.STATIC);
        final List<FuncidDef> funcidDefs = Arrays.asList(funcidDef);
        when(funcidDefChangeRecordServiceUnderTest.funcidDefRepository.saveAll(Arrays.asList(new FuncidDef()))).thenReturn(funcidDefs);

        // Configure FuncidDefChangeRecordRepository.saveAll(...).
        final FuncidDefChangeRecord funcidDefChangeRecord = new FuncidDefChangeRecord();
        funcidDefChangeRecord.setId(0);
        funcidDefChangeRecord.setCode("code");
        funcidDefChangeRecord.setGroupCode("groupCode");
        funcidDefChangeRecord.setProjectId("projectId");
        funcidDefChangeRecord.setOrigCode("origCode");
        funcidDefChangeRecord.setName("name");
        funcidDefChangeRecord.setAliasCode("aliasCode");
        funcidDefChangeRecord.setAliasName("aliasName");
        funcidDefChangeRecord.setClassCode("classCode");
        funcidDefChangeRecord.setCategory(FuncidCategory.STATIC);
        final List<FuncidDefChangeRecord> funcidDefChangeRecords = Arrays.asList(funcidDefChangeRecord);
        when(funcidDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new FuncidDefChangeRecord()))).thenReturn(funcidDefChangeRecords);

        // Run the test
        funcidDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }

    @Test
    void testUpdateStateByVersion_FuncidDefRepositoryReturnsNoItems() {
        // Setup
        when(funcidDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QFuncidDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefRepository.saveAll(Arrays.asList(new FuncidDef()))).thenReturn(Collections.emptyList());

        // Configure FuncidDefChangeRecordRepository.saveAll(...).
        final FuncidDefChangeRecord funcidDefChangeRecord = new FuncidDefChangeRecord();
        funcidDefChangeRecord.setId(0);
        funcidDefChangeRecord.setCode("code");
        funcidDefChangeRecord.setGroupCode("groupCode");
        funcidDefChangeRecord.setProjectId("projectId");
        funcidDefChangeRecord.setOrigCode("origCode");
        funcidDefChangeRecord.setName("name");
        funcidDefChangeRecord.setAliasCode("aliasCode");
        funcidDefChangeRecord.setAliasName("aliasName");
        funcidDefChangeRecord.setClassCode("classCode");
        funcidDefChangeRecord.setCategory(FuncidCategory.STATIC);
        final List<FuncidDefChangeRecord> funcidDefChangeRecords = Arrays.asList(funcidDefChangeRecord);
        when(funcidDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new FuncidDefChangeRecord()))).thenReturn(funcidDefChangeRecords);

        // Run the test
        funcidDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }

    @Test
    void testUpdateStateByVersion_FuncidDefChangeRecordRepositoryReturnsNoItems() {
        // Setup
        when(funcidDefChangeRecordServiceUnderTest.criteriaUtils.query(eq(new QFuncidDefChangeRecord("variable")), any(CriteriaExpressionParser.class), any(JacksonCriteria.class))).thenReturn(new PagedResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.classDefService.queryClass(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());
        when(funcidDefChangeRecordServiceUnderTest.funcidDefService.queryFuncid(any(JacksonCriteria.class))).thenReturn(new ListResponse<>());

        // Configure FuncidDefRepository.saveAll(...).
        final FuncidDef funcidDef = new FuncidDef();
        funcidDef.setId("id");
        funcidDef.setGroupCode("groupCode");
        funcidDef.setProjectId("projectId");
        funcidDef.setCode("code");
        funcidDef.setOrigCode("origCode");
        funcidDef.setName("name");
        funcidDef.setAliasCode("aliasCode");
        funcidDef.setAliasName("aliasName");
        funcidDef.setClassCode("classCode");
        funcidDef.setCategory(FuncidCategory.STATIC);
        final List<FuncidDef> funcidDefs = Arrays.asList(funcidDef);
        when(funcidDefChangeRecordServiceUnderTest.funcidDefRepository.saveAll(Arrays.asList(new FuncidDef()))).thenReturn(funcidDefs);

        when(funcidDefChangeRecordServiceUnderTest.repository.saveAll(Arrays.asList(new FuncidDefChangeRecord()))).thenReturn(Collections.emptyList());

        // Run the test
        funcidDefChangeRecordServiceUnderTest.updateStateByVersion(EnumVersionState.CANCEL, "version");

        // Verify the results
    }
}
